//---------------------------------------------------------------------------
//
//   MergeMorph
//
//          Copyright(C) 2015, Tark
//
//     [t
//
//---------------------------------------------------------------------------

#include <windows.h>
//#include <stdio.h>
#include <sstream>

#include "CMQObject.h"
#include "MergeMorphWindow.h"
#include "MergeMorph.h"
#include "MQSetting.h"

//---------------------------------------------------------------------------
//  MQGetPlugInID
//---------------------------------------------------------------------------
void MergeMorph::GetPlugInID(DWORD *Product, DWORD *ID)
{
	*Product = 0xE3F7A830;
	*ID = 0xBD9A1885;
}

//---------------------------------------------------------------------------
//  MQGetPlugInName
//---------------------------------------------------------------------------
const char *MergeMorph::GetPlugInName(void)
{
	// vOC
	return "MergeMorph ver.1.0 Copyright(C) 2015, Tark.";
}

//---------------------------------------------------------------------------
//  MQEnumString
//---------------------------------------------------------------------------
const char *MergeMorph::EnumString(int index)
{
	switch (index) {
	case 0: return "MergeMorph"; break;
	}
	return NULL;
}

MQBasePlugin *GetPluginClass()
{
	static MergeMorph plugin;
	return &plugin;
}

BOOL MergeMorph::Execute(int index, MQDocument doc_)
{
	switch (index) {
	case 0:
	{
		doc = cmq::CMQDocument(doc_);
		return main();
	}
	}
	return FALSE;
}

MergeMorph::MergeMorph() : doc(nullptr)
{
}

MergeMorph::~MergeMorph()
{
}

BOOL MergeMorph::main()
{
	m_baseobj = doc.getCurrentObject();
	if (!m_baseobj.isLive()) {
		return FALSE;
	}
	if (!callDialog()) {
		return FALSE;
	}

	// baseN[newmorphIuWFNg
	cmq::CMQObject newmorph = m_baseobj.clone();
	newmorph.setName(cmq::tostring(m_newmorphname));

	// morph1𔽉f
	addMorph(newmorph, m_baseobj, m_morphobj1, m_weight1);
	// morph2𔽉f
	addMorph(newmorph, m_baseobj, m_morphobj2, m_weight2);

	// newmorphǉ
	doc.insertObject(newmorph, m_baseobj);

	return TRUE;
}

// [tl
bool MergeMorph::addMorph(cmq::CMQObject &newmorph, cmq::CMQObject &baseobj, cmq::CMQObject &morphobj, double weight)
{
	if (!morphobj.isLive()) {
		return false;
	}
	cmq::CMQContainer<cmq::CMQVertex, cmq::CMQObject> vcontainer(&baseobj);
	for (auto vit = vcontainer.begin(); vit != vcontainer.end(); ++vit) {
		cmq::vindex_t vindex = vit.index();
		auto vbase = baseobj.getVertex(vindex);
		auto vmorph = morphobj.getVertex(vindex);
		if (vbase == vmorph) {
			continue;
		}
		auto vnew = newmorph.getVertex(vindex);
		vnew += (vmorph - vbase) * weight;
		newmorph.setVertex(vindex, vnew);
	}
	return true;
}

// morphobjnameobjnameɑ΂郂[ť`"elem:*-(*)"ł邩ۂ
bool MergeMorph::isMorphObject(const std::string &objname, const std::string &morphobjname)
{
	if (morphobjname.substr(0, 5) != "elem:") {
		return false;
	}
	std::string::size_type pos;
	if ((pos = morphobjname.find('-')) != std::string::npos) {
		if (objname == morphobjname.substr(pos + 1)) {
			return true;
		}
	}
	return false;
}

// orignameIuWFNg𔲂o"`:(*)-`"
std::string MergeMorph::getObjectMainName(const std::string &origname)
{
	std::string name(origname);
	std::string::size_type pos;
	if ((pos = name.find(':')) != std::string::npos) {
		name = name.substr(pos + 1);
	}
	if ((pos = name.find('-')) != std::string::npos && pos > 1) {
		name = name.substr(0, pos);
	}
	return name;
}

// elementIuWFNgXgAbv
std::vector<cmq::CMQObject> MergeMorph::getMorphlist()
{
	std::vector<cmq::CMQObject> morphlist;
	std::string basename = getObjectMainName(m_baseobj.getName());
	int vnum = m_baseobj.getVertexCount();

	int ocount = doc.getObjectCount();
	for (cmq::oindex_t oindex = 0; oindex < ocount; oindex++) {
		auto obj = doc.getObject(oindex);
		if (!obj.isLive()) {
			continue;
		}
		if (obj == m_baseobj) {
			continue;
		}
		if (!isMorphObject(basename, obj.getName())) {
			continue;
		}
		if (obj.getVertexCount() != vnum) {
			continue;
		}
		morphlist.push_back(obj);
	}
	return morphlist;
}

// ̃IuWFNg`FbN
bool MergeMorph::existObject(const std::string &name)
{
	int ocount = doc.getObjectCount();
	for (cmq::oindex_t oindex = 0; oindex < ocount; oindex++) {
		auto obj = doc.getObject(oindex);
		if (!obj.isLive()) {
			continue;
		}
		if (obj.getName() == name) {
			return true;
		}
	}
	return false;
}

// d`FbNāAǉłV[tIuWFNg𐶐
std::string MergeMorph::getNewMorphName(std::string &morph)
{
	std::string baseobjname = m_baseobj.getName();
	int count = 0;
	std::string newname = "elem:" + morph + "-" + getObjectMainName(baseobjname);
	while (existObject(newname)) {
		count++;
		std::ostringstream os;
		os << "elem:" << morph << count << "-" << getObjectMainName(baseobjname);
		newname = os.str();
	}
	return newname;
}

// _CAO
BOOL MergeMorph::callDialog()
{
	MQWindow mainwin = MQWindow::GetMainWindow();
	MergeMorphWindow dlg(mainwin, this);
	if (dlg.Execute() != MQDialog::DIALOG_OK) {
		return FALSE;
	}
	m_newmorphname = dlg.m_newmorph->GetText();
	long long uid1 = dlg.m_list1->GetItemTag(dlg.m_list1->GetCurrentIndex());
	long long uid2 = dlg.m_list2->GetItemTag(dlg.m_list2->GetCurrentIndex());
	m_morphobj1 = doc.getObjectFromUniqueID(uid1);
	m_morphobj2 = doc.getObjectFromUniqueID(uid2);
	m_weight1 = dlg.m_weight1->GetPosition() / 100.0;
	m_weight2 = dlg.m_weight2->GetPosition() / 100.0;
	return TRUE;
}

